==> exp4j-0.5.1-sources.jar: net/objecthunter/exp4j/test.java <==
package net.objecthunter.exp4j;

import net.objecthunter.exp4j.function.Function;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Arrays;
import java.util.Date;
import java.util.OptionalDouble;

public class test {
    public static void main(String[] args) {
        // 定义数学表达式
        String expressionString = "(10-1)/-2";
        //execute(expressionString,2);
        //System.out.println(removeEmojis("心有力\uD83D\uDD1D而餘額不足\n12321312+---22"));
        //testMap();
        System.out.println(execute(expressionString,null));
    }
    public static BigDecimal execute(String expressionString, Date date){
        return execute(expressionString,null,date);
    }
    public static BigDecimal execute(String expressionString, Integer decimals,Date date) {
        // 定义数学表达式
        System.out.println(expressionString);
        //String expressionString = "sum(1,2,3,-4)*avg(1,5,9)/2";
        //expressionString=expressionString.replace("sum(","(").replace(",","+");
        //expressionString=expressionString.replace("avg(","(").replace(",","+");
        try {
            //Double daysOfMonth = daysOfMonth(date==null?new Date():date);
            //expressionString = expressionString.replace("${daysOfMonth}", daysOfMonth.toString());



            Function avg = new Function("avg",0) {

                @Override
                public double apply(double... args) {
                    System.out.println("avg:"+args.length);
                    //this.setNumArguments(args.length);
                    OptionalDouble avg = Arrays.stream(args).average();
                    return avg.isPresent() ? avg.getAsDouble() : 0;
                }
            };
            Function sum = new Function("sum",0) {
                @Override
                public double apply(double... args) {
                    System.out.println("sum:"+args.length);
                    //this.setNumArguments(args.length);
                    //System.out.println( args.length);
                    return Arrays.stream(args).sum();
                }
            };
            Function first = new Function("first",0) {

                @Override
                public double apply(double... args) {
                    //this.setNumArguments(args.length);
                    //System.out.println(Arrays.stream(args).toArray().length);
                    //System.out.println( args.length);
                    if(args.length==0){
                        return 0;
                    }else {
                        return args[0];
                    }
                }
            };
            Function max = new Function("max",0) {

                @Override
                public double apply(double... args) {
                    //this.setNumArguments(args.length);
                    //System.out.println(Arrays.stream(args).toArray().length);
                    //System.out.println( args.length);
                    return Arrays.stream(args).max().isPresent()?Arrays.stream(args).max().getAsDouble():0;
                }
            };
            Function min = new Function("min",0) {

                @Override
                public double apply(double... args) {
                    //this.setNumArguments(args.length);
                    //System.out.println(Arrays.stream(args).toArray().length);
                    //System.out.println( args.length);
                    return Arrays.stream(args).min().isPresent()?Arrays.stream(args).min().getAsDouble():0;
                }
            };
            // 创建 Expression 对象
            Expression expression = new ExpressionBuilder(expressionString).
                    function(sum).functions(avg).functions(first).functions(max).functions(min)
                    .build();
            // 计算表达式的值
            double result = expression.evaluate();

            if (decimals == null) {
                return BigDecimal.valueOf(result);
            }
            // 打印结果
            //System.out.println("表达式的值为：" + BigDecimal.valueOf(result).setScale(decimals, RoundingMode.HALF_UP));

            return BigDecimal.valueOf(result).setScale(decimals, RoundingMode.HALF_UP);
        }catch (ArithmeticException e){
            //logger.error("表达式计算异常",e);
            return null;
        }catch (IllegalArgumentException ee){
            //logger.error("表达式计算异常1",ee);
            return null;
        }

    }
}

==> exp4j-0.5.2-sources.jar: net/objecthunter/exp4j/test.java <==
package net.objecthunter.exp4j;

import net.objecthunter.exp4j.function.Function;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.OptionalDouble;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class test {
    public static void main(String[] args) {
        // 定义数学表达式
        String expressionString = "sum(-23*22,-1*1)";
        expressionString=convertSumToPlus(expressionString);
        //execute(expressionString,2);
        //System.out.println(removeEmojis("心有力\uD83D\uDD1D而餘額不足\n12321312+---22"));
        //testMap();
        System.out.println(execute(expressionString,null));
    }
    public static BigDecimal execute(String expressionString, Date date){
        return execute(expressionString,null,date);
    }
    public static BigDecimal execute(String expressionString, Integer decimals,Date date) {
        // 定义数学表达式
        System.out.println(expressionString);
        //String expressionString = "sum(1,2,3,-4)*avg(1,5,9)/2";
        //expressionString=expressionString.replace("sum(","(").replace(",","+");
        //expressionString=expressionString.replace("avg(","(").replace(",","+");
        try {
            //Double daysOfMonth = daysOfMonth(date==null?new Date():date);
            //expressionString = expressionString.replace("${daysOfMonth}", daysOfMonth.toString());



            Function avg = new Function("avg",0) {

                @Override
                public double apply(double... args) {
                    System.out.println("avg:"+args.length);
                    this.setNumArguments(args.length);
                    OptionalDouble avg = Arrays.stream(args).average();
                    return avg.isPresent() ? avg.getAsDouble() : 0;
                }
            };
            Function sum = new Function("sum",0) {
                @Override
                public double apply(double... args) {

                    //this.setNumArguments(args.length);

                    //System.out.println( args.length);
                    return Arrays.stream(args).sum();
                }
            };
            Function first = new Function("first",0) {

                @Override
                public double apply(double... args) {
                    //this.setNumArguments(args.length);
                    //System.out.println(Arrays.stream(args).toArray().length);
                    //System.out.println( args.length);
                    if(args.length==0){
                        return 0;
                    }else {
                        return args[0];
                    }
                }
            };
            Function max = new Function("max",0) {

                @Override
                public double apply(double... args) {
                    //this.setNumArguments(args.length);
                    //System.out.println(Arrays.stream(args).toArray().length);
                    //System.out.println( args.length);
                    return Arrays.stream(args).max().isPresent()?Arrays.stream(args).max().getAsDouble():0;
                }
            };
            Function min = new Function("min",0) {

                @Override
                public double apply(double... args) {
                    //this.setNumArguments(args.length);
                    //System.out.println(Arrays.stream(args).toArray().length);
                    //System.out.println( args.length);
                    return Arrays.stream(args).min().isPresent()?Arrays.stream(args).min().getAsDouble():0;
                }
            };
            // 创建 Expression 对象

            Expression expression = new ExpressionBuilder(expressionString).
                    function(sum).functions(avg).functions(first).functions(max).functions(min)
                    .build();
            // 计算表达式的值
            double result = expression.evaluate();

            if (decimals == null) {
                return BigDecimal.valueOf(result);
            }
            // 打印结果
            //System.out.println("表达式的值为：" + BigDecimal.valueOf(result).setScale(decimals, RoundingMode.HALF_UP));

            return BigDecimal.valueOf(result).setScale(decimals, RoundingMode.HALF_UP);
        }catch (ArithmeticException e){
            //logger.error("表达式计算异常",e);
            return null;
        }catch (IllegalArgumentException ee){
            //logger.error("表达式计算异常1",ee);
            return null;
        }

    }
    public static String convertSumToPlus(String expression) {
        // 正则匹配 sum(...)，支持嵌套括号（简单一层）
        Pattern pattern = Pattern.compile("sum\\((.*)\\)", Pattern.CASE_INSENSITIVE);
        Matcher matcher = pattern.matcher(expression);

        StringBuffer result = new StringBuffer();
        while (matcher.find()) {
            String args = matcher.group(1); // 提取括号内的参数：23*24,12*2,33*4

            // 按逗号分割参数（注意：逗号不能在嵌套函数内，这里假设简单情况）
            String[] parts = args.split(",");
            ArrayList<String> cleanedParts = new ArrayList<>();

            for (String part : parts) {
                cleanedParts.add(part.trim());
            }

            // 用 " + " 连接所有参数
            String replacement = String.join(" + ", cleanedParts);

            // 替换原始的 sum(...) 部分
            matcher.appendReplacement(result, Matcher.quoteReplacement(replacement));
        }
        matcher.appendTail(result);

        return result.toString();
    }
}
